@@ -6,13 +6,10 @@ import numpy as np
 import numpy.linalg as la
 from scipy import signal, stats
 import ephem
-import matplotlib as mpl
-import matplotlib.pyplot as plt
 from datetime import datetime
 from datetime import timezone
 from IPython.display import display,clear_output
 #import healpy
-import gc
 import aram_tools
 
 
@@ -22,19 +19,9 @@ default_site_config  = {'site'  : 'ACT'}
  
 
 
-default_other_config = {'fov_tol' : 5e-2, 'n_dz_samp' : 16, 'n_ca_samp' : 16, 'n_st_samp' : 16}
 
-default_model_config = {'n_layers'     : 4,
-                        'min_height'   : 1000,
-                        'max_height'   : 3000,
-                        'sig_res'      : 1,
-                        'half_height'  : 1500,
-                        'atm_rms'      : 50,  
-                        'wind_speed'   : 50,
-                        'wind_bearing' : 135,
-                        'cov_type'     : 'matern',
-                        'outer_scale'  : 500,
-                        'power_law'    : -8/3}  
+
+
 
 
 # an array object describes the array of detectors. all of the arguments are dimensionful arrays of the same length 
@@ -109,26 +96,50 @@ class site():
             print('No site config specified, using the ACT site.')
             site_config = default_site_config
             
-        manual_site = np.all(np.isin(['latitude','longitude','altitude'],list(site_config)))
+        manual_site = np.all(np.isin(['location','latitude','longitude','altitude'],list(site_config)))
         
         for arg in ['site','time']:
             if not arg in list(site_config):
                 site_config[arg] = default_site_config[arg]
         if manual_site: 
-            latitude, longitude, altitude = site_config['latitude'], site_config['longitude'], site_config['altitude']
+            location, latitude, longitude, altitude = site_info['location'], site_config['latitude'], site_config['longitude'], site_config['altitude']
         else:
             if not site_config['site'] in list(self.site_dict):
                 raise Exception(f'Not a supported site. Available sites are:\n' + site_list)
             else:
                 site_info = self.site_dict[site_config['site']]
-                latitude, longitude, altitude = site_info['latitude'], site_info['longitude'], site_info['altitude']
+                self.is_auto = True
+                location, latitude, longitude, altitude = site_info['location'], site_info['latitude'], site_info['longitude'], site_info['altitude']
 
         #print(latitude, longitude, altitude)
-        
+        self.location = location
         self.site.lat, self.site.lon, self.site.elevation = str(latitude), str(longitude), altitude
         self.timestamp = site_config['time']
+        
         self.site.date = datetime.fromtimestamp(self.timestamp)
+        
+        if self.is_auto:
+            self.yday = datetime.fromtimestamp(self.timestamp).timetuple().tm_yday - 1
+            self.hour = datetime.fromtimestamp(self.timestamp).timetuple().tm_hour - 1
 
+            self.ihour = int(self.hour/3) if self.hour < 22.5 else 0
+            self.iyday = int(self.hour/7) 
+
+            with open('products/weather/norm_gen', 'rb') as handle:
+                norm_gen = pickle.load(handle)
+            with open('products/weather/mode_avg', 'rb') as handle:
+                mode_avg = pickle.load(handle)
+            with open('products/weather/mode_rms', 'rb') as handle:
+                mode_rms = pickle.load(handle)
+
+            gw = np.matmul(norm_gen[self.location][self.iyday,self.ihour],np.random.standard_normal(16)).reshape((4,33))
+            self.gw_dict = {}
+            for icol,col in enumerate(['LQV','T','U','V']):
+                gw[icol] *= mode_rms[self.location][col]
+                gw[icol] += mode_avg[self.location][col]
+                self.gw_dict[col] = gw[icol]
+            self.gw_dict['QV'] = np.exp(self.gw_dict['LQV'])
+        
 class observation():
     def __init__(self, site_config=None, obs_config=None):
         
@@ -153,14 +164,32 @@ class observation():
 
 class model():
     
-    def __init__(self,array_config=None, site_config=None, obs_config=None, model_config=None, other_config=default_other_config, verbose=False):
+    def __init__(self,array_config=None, site_config=None, obs_config=None, model_config=None, other_config=None, full_compute=False, verbose=False):
+        
+        default_model_config = {'n_layers'     : 4,
+                                'min_height'   : 1000,
+                                'max_height'   : 3000,
+                                'sig_res'      : 1,
+                                'half_height'  : 1500,
+                                'atm_rms'      : 50,  
+                                'wind_speed'   : 50,
+                                'wind_bearing' : 0,
+                                'cov_type'     : 'matern',
+                                'outer_scale'  : 500,
+                                'power_law'    : -8/3}  
+        
+        default_other_config = {'fov_tol' : 5e-2, 'n_dz_samp' : 16, 'n_ca_samp' : 16, 'n_st_samp' : 16, 'n_sam_max' : 10000}
         
         # MODEL CONFIGURATION
         if model_config==None:
             self.model_is_default = True  
             print('No model config specified, using default atmospheric model.')
             model_config = default_model_config
-        
+            
+        if other_config==None:
+            self.other_is_default = True  
+            other_config = default_other_config
+            
         self.array = array(array_config=array_config)
         self.site  = site(site_config=site_config)
         self.obs   = observation(obs_config=obs_config)
@@ -169,9 +198,9 @@ class model():
 
         self.obs.mlst0 = float(self.site.site.sidereal_time()) 
         self.obs.mlst_ = self.obs.mlst0 % (2*np.pi) + ((2*np.pi / 86400) * self.obs.t_) % (2*np.pi) # mean local sidereal time        
-        self.ca_samp = self.obs.c_az + np.linspace(-self.obs.r_az,self.obs.r_az,default_other_config['n_ca_samp'])#+ np.pi/2
-        self.dz_samp = self.array.fov_r * np.exp(1j*np.linspace(0,2*np.pi,default_other_config['n_dz_samp']+1)[:-1])
-        self.st_samp = np.linspace(self.obs.mlst_[0],self.obs.mlst_[-1],default_other_config['n_st_samp'])
+        self.ca_samp = self.obs.c_az + np.linspace(-self.obs.r_az,self.obs.r_az,other_config['n_ca_samp'])#+ np.pi/2
+        self.dz_samp = self.array.fov_r * np.exp(1j*np.linspace(0,2*np.pi,other_config['n_dz_samp']+1)[:-1])
+        self.st_samp = np.linspace(self.obs.mlst_[0],self.obs.mlst_[-1],other_config['n_st_samp'])
         self.dx_samp, self.dy_samp = np.real(self.dz_samp), np.imag(self.dz_samp)
         self.fov_simp = sp.spatial.Delaunay(np.vstack([self.dx_samp, self.dy_samp]).T, incremental=False, qhull_options=None)
 
@@ -208,25 +237,34 @@ class model():
             exec(f'self.{arg} = thing')
                 
         self.n_layers = len(self.heights)
+        
+        if self.site.is_auto:
+            self.weather = {}
+            for col in ['QV','T','U','V']:
+                self.weather[col] = np.interp(self.heights,np.arange(0,6600,200),self.site.gw_dict[col]) 
+            z = self.weather['V'] + 1j*self.weather['U']
+            self.weather['ws'], self.weather['wa'] = np.abs(z), np.angle(z)
+            self.wind_bearing = self.weather['wa']
+            self.wind_speed = self.weather['ws']
+        
+        else:
                 
-        if isinstance(model_config['wind_speed'], np.ndarray):
-            self.wind_speed = model_config['wind_speed']
-        else:
-            self.wind_speed = model_config['wind_speed'] * np.ones(self.n_layers)
-            
-        if isinstance(model_config['wind_bearing'], np.ndarray):
-            self.wind_bearing = np.radians(model_config['wind_bearing'])
-        else:
-            self.wind_bearing = np.radians(model_config['wind_bearing']) * np.ones(self.n_layers)
-            
-        if not (len(self.wind_speed) == self.n_layers and len(self.wind_bearing) == self.n_layers):
-            raise Exception('Shape mismatch in the heights and the wind velocities.')
-            
+            if isinstance(model_config['wind_speed'], np.ndarray):
+                self.wind_speed = model_config['wind_speed']
+            else:
+                self.wind_speed = model_config['wind_speed'] * np.ones(self.n_layers)
+
+            if isinstance(model_config['wind_bearing'], np.ndarray):
+                self.wind_bearing = np.radians(model_config['wind_bearing'])
+            else:
+                self.wind_bearing = np.radians(model_config['wind_bearing']) * np.ones(self.n_layers)
+
+            if not (len(self.wind_speed) == self.n_layers and len(self.wind_bearing) == self.n_layers):
+                raise Exception('Shape mismatch in the heights and the wind velocities.')
+
         if model_config['cov_type'] == 'matern':
             self.r0, self.nu = model_config['outer_scale'], (-1-model_config['power_law'])/2
             self.C = lambda r: 2**(1-self.nu)/sp.special.gamma(self.nu)*sp.special.kv(self.nu,r/self.r0+1e-10)*(r/self.r0+1e-10)**self.nu
-            
-        #print('made model')
         
         self.array.wavelength = 2.998e8 / self.array.band
                 
@@ -236,30 +274,30 @@ class model():
         self.ang_sig_ = aram_tools.gaussian_beam(self.depths[None,:],self.w0[:,None],l=self.array.wavelength[:,None],n=1.003)
         self.phy_sig_ = self.ang_sig_.min(axis=0) * self.depths
         
-        self.atm_res_ = self.phy_sig_ * model_config['sig_res']
+        self.atm_res_ = self.phy_sig_ * model_config['sig_res'] * (2*np.sqrt(2*np.log(2)))
         self.d_orth_  = self.atm_res_ 
-        self.sim_dt   = np.maximum((self.atm_res_ / self.wind_speed).min(),self.obs.dt)
-        self.d_para_  = self.sim_dt * self.wind_speed
+        self.d_para_  = self.atm_res_ 
+        self.sim_dt_  = self.atm_res_ / self.wind_speed
         
         # time and angle fields for the simulation
-        self.sim_t_    = self.obs.t_[0] + np.arange(-self.sim_dt,self.obs.duration+self.sim_dt,self.sim_dt)
-        self.sim_mlst_ = np.interp(self.sim_t_,self.obs.t_,self.obs.mlst_)
-        self.sim_T = len(self.sim_t_); self.sim_f_ = np.fft.fftfreq(self.sim_T,self.sim_dt)
-        self.sim_c_az_ = np.interp(self.sim_t_,self.obs.t_,self.obs.c_az_)
-        self.sim_c_el_ = np.interp(self.sim_t_,self.obs.t_,self.obs.c_el_)
+        self.sim_t_    = [self.obs.t_[0] + np.arange(-dt,self.obs.duration+dt, dt) for dt in self.sim_dt_]
+        self.sim_mlst_ = np.interp(self.sim_t_[0],self.obs.t_,self.obs.mlst_)
+        self.sim_T_    = [len(sim_t_) for sim_t_ in self.sim_t_]; self.sim_f_ = [np.fft.fftfreq(T,dt) for T,dt in zip(self.sim_T_,self.sim_dt_)]
+        self.tot_sim_T = np.sum(self.sim_T_)
+        self.sim_c_az_ = [np.interp(t_,self.obs.t_,self.obs.c_az_) for t_ in self.sim_t_]
+        self.sim_c_el_ = [np.interp(t_,self.obs.t_,self.obs.c_el_) for t_ in self.sim_t_]
 
-        self.sim_az_, self.sim_el_  = aram_tools.xy_to_ae(self.array.x[:,None],self.array.y[:,None],self.sim_c_az_[None,:],self.sim_c_el_[None,:])
-        self.sim_dx_, self.sim_dy_  = aram_tools.ae_to_xy(self.sim_az_, self.sim_el_, self.obs.c_az, self.obs.c_el)
+        self.sim_az_, self.sim_el_ = list(zip(*[aram_tools.xy_to_ae(self.array.x[:,None],self.array.y[:,None],caz[None,:],cel[None,:]) for caz,cel in zip(self.sim_c_az_,self.sim_c_el_)]))
+        #self.sim_az_, self.sim_el_ = list(zip(*[aram_tools.xy_to_ae(self.array.x[:,None],self.array.y[:,None],caz[None,:],cel[None,:]) for caz,cel in zip(self.sim_c_az_,self.sim_c_el_)]))
+        #
         #self.sim_ra_, self.sim_dec_ = aram_tools.ae_to_rd(self.sim_az_, self.sim_el_,self.sim_mlst_[None,:],self.site.site.lat)
-        self.fov_az_, self.fov_el_  = aram_tools.xy_to_ae(self.dx_samp[:,None],self.dy_samp[:,None],self.sim_c_az_[None,:],self.sim_c_el_[None,:])
-    
+        #self.fov_az_, self.fov_el_  = [aram_tools.xy_to_ae(dx[:,None],dy[:,None],caz[None,:],cel[None,:]) for dx,dy,caz,cel in zip(self.dx_samp,self.dy_samp,self.sim_c_az_,self.sim_c_el_)]
+        self.fov_az_, self.fov_el_  = aram_tools.xy_to_ae(self.dx_samp[:,None],self.dy_samp[:,None],self.sim_c_az_[0][None,:],self.sim_c_el_[0][None,:])    
         self.sim_min_az_, self.sim_max_az_ = self.fov_az_.min(axis=0), self.fov_az_.max(axis=0)
         self.sim_min_el_, self.sim_max_el_ = self.fov_el_.min(axis=0), self.fov_el_.max(axis=0)
         
         del self.fov_az_, self.fov_el_
         gc.collect()
-                
-        n_, n_sam, n_gen = 0, 0, 0
         self.lay_z_, self.lay_h_ = [], []
         self.gen_z_, self.gen_h_ = [], []
         self.n_para_, self.n_orth_ = [], []
@@ -267,7 +305,13 @@ class model():
         self.sam_i_ = []
         
         # this is messy, but necessary
+        if verbose:
+            print('  # | height [m] | res. [m] | n_para | n_ortho |  n_total | n_sam | n_gen ')
+
+        self.good_layer_ = np.ones(self.n_layers).astype(bool)
+        
         for i_h,h in enumerate(self.heights):
+            
             adj_bear = 3*np.pi/2 + self.wind_bearing[i_h]
             self.rot_ch_z    = h * self.rel_z * np.exp(1j*adj_bear)
             ch_para, ch_orth = np.real(self.rot_ch_z),np.imag(self.rot_ch_z)
@@ -288,13 +332,25 @@ class model():
                 i_para_.append(np.repeat(i_para,len(i_orth_[-1])).astype(int))
             i_max  = np.maximum(0,np.max(i_para_[-1])) 
             self.sam_i_.append((np.concatenate(i_para_),np.concatenate(i_orth_)))
-            n_    += len(z_.ravel())
-            n_sam += len(self.sam_i_[-1][0].ravel())
-            n_gen += len(self.gen_z_[-1].ravel())
-            if verbose:
-                print(z_.shape,len(self.sam_i_[-1][0].ravel()), len(self.gen_z_[-1].ravel()))
+            n_    = len(z_.ravel())
+            n_sam = len(self.sam_i_[-1][0].ravel())
+            n_gen = len(self.gen_z_[-1].ravel())
             
-        if not self.default_interp_method == 'rectilinear':
+            if n_sam > other_config['n_sam_max']:
+                self.good_layer_[i_h] = False
+            
+            if verbose:
+                if self.good_layer_[i_h]:
+                    print(f' {i_h:>2} | {h:>10.02f} | {self.atm_res_[i_h]:>8.02f} | {z_.shape[0]:>6.0f} | {z_.shape[1]:>7.0f} | {n_:>7.02e} | {n_sam:>5.0f} | {n_gen:>5.0f}')
+                else:
+                    print(f' {i_h:>2} | {h:>10.02f} | WARNING: LAYER EXCLUDED')
+
+            
+        if (not self.default_interp_method == 'rectilinear') or full_compute:
+            
+            print('building trees...')
+            
+            self.sim_dx_, self.sim_dy_  = list(zip(*[aram_tools.ae_to_xy(az,el,caz,cel) for az,el,caz,cel in zip(self.sim_az_, self.sim_el_, self.obs.c_az, self.obs.c_el)]))
             self.lay_x_, self.lay_y_ = [np.real(z)  for z in self.lay_z_], [np.imag(z) for z in self.lay_z_]
             self.lay_a_, self.lay_e_ = [np.pi/2 - np.angle(z) for z in self.lay_z_], [np.pi/2 - np.arctan(np.abs(z/h)) for z,h in zip(self.lay_z_,self.heights)]
             self.lay_dx_, self.lay_dy_, self.lay_dz_ = [], [], []
@@ -305,6 +361,8 @@ class model():
             self.static_ckdt = [sp.spatial.cKDTree(np.vstack([x.ravel(),y.ravel()]).T) for x,y in zip(self.lay_dx_, self.lay_dy_)]
 
         # this is too 
+        if verbose:
+            print('computing transition matrices...')
         sam_args = [np.c_[np.real(lz[si]),np.imag(lz[si]),lh[si]] for lz,lh,si in zip(self.lay_z_,self.lay_h_,self.sam_i_)]
         gen_args = [np.c_[np.real(gz),np.imag(gz),gh]             for gz,gh    in zip(self.gen_z_,self.gen_h_)]
         lay_v_   = [np.zeros(lz.shape) for lz in self.lay_z_]
@@ -314,43 +372,41 @@ class model():
         self.cgen_ = [aram_tools.make_covariance_matrix(self.C,np.real(gz),np.imag(gz),gh) for gz,gh in zip(self.gen_z_,self.gen_h_)]
         self.A_ = [np.matmul(csam,prec) for prec,csam in zip(self.prec_,self.csam_)]
         self.B_ = [aram_tools.msqrt(cgen-np.matmul(A,csam.T)) for A,cgen,csam in zip(self.A_,self.cgen_,self.csam_)]
-        del self.prec_, self.csam_, self.cgen_
-        gc.collect()
         
         print(f'done! total atmospheric points : {n_}')
         
-    def atmosphere_timestep(self):
+    def atmosphere_timestep(self,i_layer):
         
-        gen_v_ = [np.matmul(A,lv[si]) + np.matmul(B,np.random.standard_normal(B.shape[0])) for A,B,lv,si in zip(self.A_,self.B_,self.lay_v_,self.sam_i_)]
-        for igv,gv in enumerate(gen_v_):
-            self.lay_v_[igv] = np.r_[gv[None,:],self.lay_v_[igv][:-1]]
+        self.lay_v_[i_layer] = np.r_[(np.matmul(self.A_[i_layer],self.lay_v_[i_layer][self.sam_i_[i_layer]])
+                                 + np.matmul(self.B_[i_layer],np.random.standard_normal(self.B_[i_layer].shape[0])))[None,:],self.lay_v_[i_layer][:-1]]
 
     def generate_atmosphere(self,blurred=True):
 
         self.lay_v_ = [np.zeros(lz.shape) for lz in self.lay_z_]
-        n_init = 2*np.concatenate([si[0] for si in self.sam_i_]).max()
-        n_ts   = self.sim_T + np.max(self.n_para_)
-        self.gen_data = [np.zeros((n_ts,v.shape[1])) for v in self.lay_v_]
+        n_init_ = [si[0].max() for si in self.sam_i_]
+        n_ts_   = [T+n_para for T,n_para in zip(self.sim_T_,self.n_para_)]
+        tot_n_init, tot_n_ts = np.sum(n_init_), np.sum(n_ts_)
+        self.gen_data = [np.zeros((n_ts,v.shape[1])) for n_ts,v in zip(n_ts_,self.lay_v_)]
         
-        prog = aram_tools.progress_bar(n_total=n_init,n_per=16,task='INITIALIZING...')
-        for i_init in range(n_init):
-            prog.update(i_init+1)
-            self.atmosphere_timestep()
+        prog = aram_tools.progress_bar(n_total=tot_n_init,n_per=64,task='INITIALIZING...')
+        tot_i_init = 0
+        for i_layer, n_init in enumerate(n_init_):
+            for i_init in range(n_init):
+                prog.update(tot_i_init+1); tot_i_init += 1
+                self.atmosphere_timestep(i_layer)
             
-        prog = aram_tools.progress_bar(n_total=n_ts,n_per=16,task='GENERATING...')
-        for i_ts in range(n_ts):
-            prog.update(i_ts+1)
-            self.atmosphere_timestep()
-            for i_v,v in enumerate(self.lay_v_):
-                self.gen_data[i_v][i_ts] = v[0]
-        et = time.time()
-        
+        prog = aram_tools.progress_bar(n_total=tot_n_ts,n_per=64,task='GENERATING...')
+        tot_i_ts = 0
+        for i_layer,n_ts in enumerate(n_ts_):
+            for i_ts in range(n_ts):
+                prog.update(tot_i_ts+1); tot_i_ts += 1
+                self.atmosphere_timestep(i_layer)
+                self.gen_data[i_layer][i_ts] = self.lay_v_[i_layer][0]
+
         if blurred:
             for i_h, h in enumerate(self.heights):
                 self.gen_data[i_h] = sp.ndimage.gaussian_filter1d(self.gen_data[i_h],axis=0,sigma=self.phy_sig_[i_h] / self.d_orth_[i_h])
                 self.gen_data[i_h] = sp.ndimage.gaussian_filter1d(self.gen_data[i_h],axis=1,sigma=self.phy_sig_[i_h] / self.d_para_[i_h])
-                
-    
 
     def simulate(self, do_atmosphere=False, 
                        do_cmb=False, 
@@ -366,36 +422,35 @@ class model():
         gaussian = lambda offset, sig : np.exp(-.5*np.square(offset/sig))
         
         if do_atmosphere:
-            h_atm_sim_data  = np.zeros((self.n_layers,self.array.n,self.sim_T))
+            h_atm_sim_data = [np.zeros((self.array.n,sim_T)) for sim_T in self.sim_T_]
             if interp_method=='rectilinear':
+                
                 self.generate_atmosphere(blurred=True)
-                prog = aram_tools.progress_bar(n_total=self.sim_T,n_per=16,task='OBSERVING...')
-                for i_t, t in enumerate(self.sim_t_):
-                    prog.update(i_t+1)
-                    for i_h, (h, p_, o_) in enumerate(zip(self.heights,self.lay_para_,self.lay_orth_)): 
-
-                        rel_z_ = h*np.exp(1j*(np.pi/2-self.sim_az_[:,i_t])) / np.tan(self.sim_el_[:,i_t])*np.exp(1j*(3*np.pi/2 + self.wind_bearing[i_h]))
+                prog = aram_tools.progress_bar(n_total=self.tot_sim_T,n_per=64,task='OBSERVING...'); tot_i_t = 0
+                for i_h, (sim_t_, sim_az_, sim_el_, h, p_, o_) in enumerate(zip(self.sim_t_, self.sim_az_, self.sim_el_, 
+                                                                                self.heights, self.lay_para_, self.lay_orth_)):
+                    for i_t, t in enumerate(sim_t_):
+                        prog.update(tot_i_t+1); tot_i_t += 1
+                        rel_z_ = h*np.exp(1j*(np.pi/2-sim_az_[:,i_t])) / np.tan(sim_el_[:,i_t])*np.exp(1j*(3*np.pi/2 + self.wind_bearing[i_h]))
                         rel_p_, rel_o_ = np.real(rel_z_), np.imag(rel_z_)
                         ip_min, ip_max = np.where(p_ < rel_p_.min())[0][-1], np.where(p_ > rel_p_.max())[0][0] + 1
                         io_min, io_max = np.where(o_ < rel_o_.min())[0][-1], np.where(o_ > rel_o_.max())[0][0] + 1
                         i_p = (p_ > rel_p_.min()) & (p_ < rel_p_.max())
                         i_o = (o_ > rel_o_.min()) & (o_ < rel_o_.max())
                         atm_interp = sp.interpolate.RegularGridInterpolator((p_[ip_min:ip_max], o_[io_min:io_max]), self.gen_data[i_h][i_t:i_t+self.n_para_[i_h]][ip_min:ip_max,io_min:io_max])
-                        h_atm_sim_data[i_h,:,i_t] = atm_interp((rel_p_, rel_o_))
+                        h_atm_sim_data[i_h][:,i_t] = atm_interp((rel_p_, rel_o_))
 
             if interp_method=='tree_gaussian':
                 self.generate_atmosphere(blurred=False)
                 prog = aram_tools.progress_bar(n_total=self.sim_T,n_per=16,task='OBSERVING...')
-                for i_t, t in enumerate(self.sim_t_):
-                    prog.update(i_t+1)
-                    for i_h, kdt in enumerate(self.static_ckdt): 
-
-                        self.dist_, self.index_ = kdt.query(np.stack([mod.sim_dx_[:,i_t], mod.sim_dy_[:,i_t]]).T, k=sky_per_det)
+                for i_h, (kdt, sim_t_, dx_, dy_) in enumerate(zip(self.static_ckdt, self.sim_t_, self.sim_dx_, self.sim_dy_)): 
+                    for i_t, t in enumerate(self.sim_t_):
+                        prog.update(i_t+1)
+                        self.dist_, self.index_ = kdt.query(np.stack([dx_[:,i_t], dy_[:,i_t]]).T, k=sky_per_det)
                         self.weights  = gaussian(self.dist_,self.ang_sig_[:,i_h][:,None])
                         self.weights /= self.weights.sum(axis=1)[:,None]
                         h_atm_sim_data[i_h,:,i_t] = (self.weights*self.gen_data[i_h][i_t:i_t+self.n_para_[i_h]].ravel()[self.index_]).sum(axis=1)
-                    
-                        
+
             if interp_method=='brute_gaussian':
                 self.generate_atmosphere(blurred=False)
                 prog = aram_tools.progress_bar(n_total=self.sim_T,n_per=16,task='OBSERVING...')
@@ -417,10 +472,12 @@ class model():
                         #weights /= weights.sum(axis=1)[:,None]
                         #h_atm_sim_data[i_h,:,i_t] = (self.gen_data[i_h][i_t:i_t+self.n_para_[i_h]][self.in_fov][index_]*weights).sum(axis=1)
 
-            var_prof     = np.exp(-2*self.heights/self.half_height)
-            var_prof    *= np.square(self.atm_rms) / var_prof.sum() 
-            atm_sim_data = (h_atm_sim_data * np.sqrt(var_prof[:,None,None])).sum(axis=0) / self.sim_el_[:,0][:,None]
-            atm_data     = np.concatenate([sp.interpolate.interp1d(self.sim_t_,atm_sim_data[i_det],kind='quadratic')(self.obs.t_)[None,:] for i_det in range(self.array.n)])
+            h_atm_data = np.concatenate([np.concatenate([sp.interpolate.interp1d(sim_t_,atm_sim_data[i_det],kind='quadratic')(self.obs.t_)[None,:] for i_det in range(self.array.n)])[None,:,:] 
+                                         for sim_t_, atm_sim_data in zip(self.sim_t_,h_atm_sim_data)])
+            var_prof  = np.exp(-2*self.heights/self.half_height)
+            var_prof *= np.square(self.atm_rms) / var_prof.sum() 
+            atm_data  = (h_atm_data * np.sqrt(var_prof[:,None,None])).sum(axis=0) / self.sim_el_[0][:,0][:,None]
+            
         
         '''
         
